@@ -15,6 +15,8 @@
 #include "logging.h"
 #include "stopoverreply.h"
 #include "stopoverrequest.h"
+#include "tripreply.h"
+#include "triprequest.h"
 #include "vehiclelayoutrequest.h"
 #include "vehiclelayoutreply.h"
 #include "datatypes/attributionutil_p.h"
@@ -35,7 +37,6 @@
 #include "backends/hafasmgatebackend.h"
 #include "backends/hafasquerybackend.h"
 #include "backends/ivvassbackend.h"
-#include "backends/motisbackend.h"
 #include "backends/motis2backend.h"
 #include "backends/navitiabackend.h"
 #include "backends/oebbbackend.h"
@@ -45,6 +46,7 @@
 #include "backends/ltglinkbackend.h"
 #include "gbfs/gbfsbackend.h"
 
+#include <QCoreApplication>
 #include <QDirIterator>
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -74,27 +76,29 @@ static inline void initResources() {
 namespace KPublicTransport {
 class ManagerPrivate {
 public:
-    QNetworkAccessManager* nam();
+    [[nodiscard]] QNetworkAccessManager* nam();
     void loadNetworks();
-    std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &obj);
+    [[nodiscard]] std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &obj);
     template <typename Backend, typename Backend2, typename ...Backends>
-    static std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &backendType, const QJsonObject &obj);
-    template <typename Backend> std::unique_ptr<AbstractBackend>
-    static loadNetwork(const QJsonObject &backendType, const QJsonObject &obj);
+    [[nodiscard]] static std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &backendType, const QJsonObject &obj);
+    template <typename Backend>
+    [[nodiscard]] static std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &backendType, const QJsonObject &obj);
     template <typename T>
-    static std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &obj);
+    [[nodiscard]] static std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &obj);
 
-    template <typename RequestT> bool shouldSkipBackend(const Backend &backend, const RequestT &req) const;
+    template <typename RequestT>
+    [[nodiscard]] bool shouldSkipBackend(const Backend &backend, const RequestT &req) const;
 
     void resolveLocation(LocationRequest &&locReq, const AbstractBackend *backend, const std::function<void(const Location &loc)> &callback);
-    bool queryJourney(const AbstractBackend *backend, const JourneyRequest &req, JourneyReply *reply);
-    bool queryStopover(const AbstractBackend *backend, const StopoverRequest &req, StopoverReply *reply);
+    [[nodiscard]] bool queryJourney(const AbstractBackend *backend, const JourneyRequest &req, JourneyReply *reply);
+    [[nodiscard]] bool queryStopover(const AbstractBackend *backend, const StopoverRequest &req, StopoverReply *reply);
 
-    template <typename RepT, typename ReqT> RepT* makeReply(const ReqT &request);
+    template <typename RepT, typename ReqT>
+    [[nodiscard]] RepT* makeReply(const ReqT &request);
 
     void readCachedAttributions();
 
-    int queryLocationOnBackend(const LocationRequest &req, LocationReply *reply, const Backend &backend);
+    [[nodiscard]] int queryLocationOnBackend(const LocationRequest &req, LocationReply *reply, const Backend &backend);
 
     Manager *q = nullptr;
     QNetworkAccessManager *m_nam = nullptr;
@@ -110,7 +114,7 @@ public:
     bool m_backendsEnabledByDefault = true;
 
 private:
-    bool shouldSkipBackend(const Backend &backend) const;
+    [[nodiscard]] bool shouldSkipBackend(const Backend &backend) const;
 };
 }
 
@@ -120,7 +124,7 @@ QNetworkAccessManager* ManagerPrivate::nam()
         m_nam = new QNetworkAccessManager(q);
         m_nam->setRedirectPolicy(QNetworkRequest::NoLessSafeRedirectPolicy);
         m_nam->setStrictTransportSecurityEnabled(true);
-        m_nam->enableStrictTransportSecurityStore(true, QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/org.kde.kpublictransport/hsts/"));
+        m_nam->enableStrictTransportSecurityStore(true, QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + "/org.kde.kpublictransport/hsts/"_L1);
     }
     return m_nam;
 }
@@ -138,6 +142,7 @@ void ManagerPrivate::loadNetworks()
 #endif
     searchDirs.push_back(u":/"_s);
 
+    std::vector<Attribution> attributions;
     for (const auto &searchDir : searchDirs) {
         QDirIterator it(searchDir + "/org.kde.kpublictransport/networks"_L1, {u"*.json"_s}, QDir::Files);
         while (it.hasNext()) {
@@ -166,7 +171,7 @@ void ManagerPrivate::loadNetworks()
                 net->setBackendId(id);
                 net->init();
                 if (!net->attribution().isEmpty()) {
-                    m_attributions.push_back(net->attribution());
+                    attributions.push_back(net->attribution());
                 }
 
                 auto b = BackendPrivate::fromJson(doc.object());
@@ -182,13 +187,21 @@ void ManagerPrivate::loadNetworks()
         return lhs.identifier() < rhs.identifier();
     });
 
-    AttributionUtil::sort(m_attributions);
+    AttributionUtil::sort(attributions);
+    if (m_attributions.empty()) {
+        // first load
+        m_attributions = std::move(attributions);
+    } else {
+        // reload
+        AttributionUtil::merge(m_attributions, attributions);
+    }
+
     qCDebug(Log) << m_backends.size() << "public transport network configurations loaded";
 }
 
 std::unique_ptr<AbstractBackend> ManagerPrivate::loadNetwork(const QJsonObject &obj)
 {
-    const auto type = obj.value(QLatin1String("type")).toObject();
+    const auto type = obj.value("type"_L1).toObject();
     // backends need to be topologically sorted according to their preference/priority here
     return loadNetwork<
         NavitiaBackend,
@@ -201,7 +214,6 @@ std::unique_ptr<AbstractBackend> ManagerPrivate::loadNetwork(const QJsonObject &
         EfaBackend,
         IvvAssBackend,
         OpenJourneyPlannerBackend,
-        MotisBackend,
         Motis2Backend,
         GBFSBackend,
         AccessibilityCloudBackend,
@@ -231,7 +243,7 @@ std::unique_ptr<AbstractBackend> ManagerPrivate::loadNetwork(const QJsonObject &
 
 static void applyBackendOptions(AbstractBackend *backend, const QMetaObject *mo, const QJsonObject &obj)
 {
-    const auto opts = obj.value(QLatin1String("options")).toObject();
+    const auto opts = obj.value("options"_L1).toObject();
     for (auto it = opts.begin(); it != opts.end(); ++it) {
         const auto idx = mo->indexOfProperty(it.key().toUtf8().constData());
         if (idx < 0) {
@@ -256,11 +268,11 @@ static void applyBackendOptions(AbstractBackend *backend, const QMetaObject *mo,
         }
     }
 
-    const auto attrObj = obj.value(QLatin1String("attribution")).toObject();
+    const auto attrObj = obj.value("attribution"_L1).toObject();
     const auto attr = Attribution::fromJson(attrObj);
     backend->setAttribution(attr);
 
-    const auto tzId = obj.value(QLatin1String("timezone")).toString();
+    const auto tzId = obj.value("timezone"_L1).toString();
     if (!tzId.isEmpty()) {
         QTimeZone tz(tzId.toUtf8());
         if (tz.isValid()) {
@@ -270,7 +282,7 @@ static void applyBackendOptions(AbstractBackend *backend, const QMetaObject *mo,
         }
     }
 
-    const auto langArray = obj.value(QLatin1String("supportedLanguages")).toArray();
+    const auto langArray = obj.value("supportedLanguages"_L1).toArray();
     QStringList langs;
     langs.reserve(langArray.size());
     std::transform(langArray.begin(), langArray.end(), std::back_inserter(langs), [](const auto &v) { return v.toString(); });
@@ -495,6 +507,8 @@ Manager::Manager(QObject *parent)
     }
 
     Cache::expire();
+
+    QCoreApplication::instance()->installEventFilter(this);
 }
 
 Manager::~Manager() = default;
@@ -625,6 +639,11 @@ JourneyReply* Manager::queryJourney(const JourneyRequest &req) const
     if (req.downloadAssets()) {
         reply->addAttributions(AssetRepository::instance()->attributions());
     }
+
+    // FIXME this is not correct for negative cache hits!
+    //if (pendingOps == 0) {
+    //    reply->addError(Reply::NoBackend, u"No viable backend found."_s);
+    //}
     reply->setPendingOps(pendingOps);
     return reply;
 }
@@ -701,6 +720,11 @@ StopoverReply* Manager::queryStopover(const StopoverRequest &req) const
     if (req.downloadAssets()) {
         reply->addAttributions(AssetRepository::instance()->attributions());
     }
+
+    // FIXME this is not correct for negative cache hits!
+    //if (pendingOps == 0) {
+    //    reply->addError(Reply::NoBackend, u"No viable backend found."_s);
+    //}
     reply->setPendingOps(pendingOps);
     return reply;
 }
@@ -787,14 +811,144 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
             break;
         }
     }
+
+    // FIXME this is not correct for negative cache hits!
+    //if (pendingOps == 0) {
+    //    reply->addError(Reply::NoBackend, u"No viable backend found."_s);
+    //}
     reply->setPendingOps(pendingOps);
     return reply;
 }
 
+// Check whether a given JourneySection matches a trip request in fallback journey query mode
+// This is essentially the same as JourneySection::isSame, with one special case for accepting
+// all specialized modes for an requested generic mode. Line::isSame takes care of most of this
+// on its own already, we just need to handle all rail-based modes explicitly here.
+[[nodiscard]] static bool isSameTrip(KPublicTransport::JourneySection req, const KPublicTransport::JourneySection &res)
+{
+    if (req.route().line().mode() == Line::Train && Line::modeIsRailBound(res.route().line().mode())) {
+        auto route = req.route();
+        auto line = route.line();
+        line.setMode(res.route().line().mode());
+        route.setLine(line);
+        req.setRoute(route);
+    }
+    return JourneySection::isSame(req, res);
+}
+
+TripReply* Manager::queryTrip(const TripRequest &req) const
+{
+    auto reply = d->makeReply<TripReply>(req);
+    int pendingOps = 0;
+
+    // validate input
+    if (!req.isValid()) {
+        reply->addError(Reply::InvalidRequest, {});
+        reply->setPendingOps(pendingOps);
+        return reply;
+    }
+
+    d->loadNetworks();
+
+    // try to find a viable backend that can do a trip query directly
+    QSet<QString> triedBackends;
+    for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
+        const auto checkBackend = [&](const Backend &backend, bool bothLocationMatch) {
+            if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
+                return;
+            }
+            const auto coverage = backend.coverageArea(coverageType);
+            if (coverage.isEmpty()) {
+                return;
+            }
+
+            if (bothLocationMatch) {
+                if (!coverage.coversLocation(req.journeySection().from()) || !coverage.coversLocation(req.journeySection().to())) {
+                    return;
+                }
+            } else {
+                if (!coverage.coversLocation(req.journeySection().from()) && !coverage.coversLocation(req.journeySection().to())) {
+                    return;
+                }
+            }
+
+            triedBackends.insert(backend.identifier());
+
+            if (BackendPrivate::impl(backend)->queryTrip(req, reply, d->nam())) {
+                ++pendingOps;
+            }
+        };
+
+        // look for coverage areas which contain both locations first
+        for (const auto &backend: d->m_backends) {
+            checkBackend(backend, true);
+        }
+        if (pendingOps) {
+            break;
+        }
+
+        // if we didn't find one, try with just a single one
+        for (const auto &backend: d->m_backends) {
+            checkBackend(backend, false);
+        }
+        if (pendingOps) {
+            break;
+        }
+    }
+
+    // emulate a trip query via a journey query
+    if (pendingOps == 0) {
+        JourneyRequest jnyReq(req.journeySection().from(), req.journeySection().to());
+        // start searching slightly earlier, so leading walking section because our coordinates
+        // aren't exactly at the right spot wont make the routing service consider the train we
+        // are looking for as impossible to reach on time
+        jnyReq.setDateTime(req.journeySection().scheduledDepartureTime().addSecs(-600));
+        jnyReq.setDateTimeMode(JourneyRequest::Departure);
+        jnyReq.setIncludeIntermediateStops(true);
+        jnyReq.setIncludePaths(true);
+        jnyReq.setModes(JourneySection::PublicTransport);
+        jnyReq.setBackendIds(req.backendIds());
+        auto jnyReply = queryJourney(jnyReq);
+        jnyReply->setParent(reply);
+        connect(jnyReply, &Reply::finished, reply, [jnyReply, reply]() {
+            jnyReply->deleteLater();
+            if (jnyReply->error() != Reply::NoError) {
+                reply->addError(jnyReply->error(), jnyReply->errorString());
+                return;
+            }
+            for (const auto &journey : jnyReply->result()) {
+                if (std::ranges::count_if(journey.sections(), [](const auto &sec) { return sec.mode() == JourneySection::PublicTransport; }) != 1) {
+                    continue;
+                }
+                const auto it = std::ranges::find_if(journey.sections(), [](const auto &sec) {
+                    return sec.mode() == JourneySection::PublicTransport;
+                });
+                assert(it != journey.sections().end());
+                qCDebug(Log) << "Got journey information:" << (*it).route().line().name() << (*it).scheduledDepartureTime();
+                if (isSameTrip(reply->request().journeySection(), *it)) {
+                    qCDebug(Log) << "Found journey information:" << (*it).route().line().name() << (*it).expectedDeparturePlatform() << (*it).expectedDepartureTime();
+                    reply->addAttributions(jnyReply->attributions());
+                    reply->addResult(nullptr, JourneySection(*it));
+                    return;
+                }
+            }
+
+            reply->addError(Reply::NotFoundError, u"Not found."_s);
+        });
+
+        reply->setPendingOps(1);
+    } else {
+        reply->setPendingOps(pendingOps);
+    }
+
+    return reply;
+}
+
 VehicleLayoutReply* Manager::queryVehicleLayout(const VehicleLayoutRequest &req) const
 {
     auto reply = d->makeReply<VehicleLayoutReply>(req);
     int pendingOps = 0;
+    int negativeCacheHit = 0;
 
     // validate input
     if (!req.isValid()) {
@@ -819,6 +973,7 @@ VehicleLayoutReply* Manager::queryVehicleLayout(const VehicleLayoutRequest &req)
             auto cache = Cache::lookupVehicleLayout(backend.identifier(), req.cacheKey());
             switch (cache.type) {
                 case CacheHitType::Negative:
+                    ++negativeCacheHit;
                     qCDebug(Log) << "Negative cache hit for backend" << backend.identifier();
                     break;
                 case CacheHitType::Positive:
@@ -842,10 +997,23 @@ VehicleLayoutReply* Manager::queryVehicleLayout(const VehicleLayoutRequest &req)
         }
     }
 
+    if (pendingOps == 0 && negativeCacheHit == 0) {
+        reply->addError(Reply::NoBackend, u"No viable backend found."_s);
+    }
     reply->setPendingOps(pendingOps);
     return reply;
 }
 
+void Manager::reload()
+{
+    if (d->m_backends.empty()) { // not loaded yet, nothing to do
+        return;
+    }
+    d->m_backends.clear();
+    d->loadNetworks();
+    Q_EMIT backendsChanged();
+}
+
 const std::vector<Attribution>& Manager::attributions() const
 {
     d->loadNetworks();
@@ -955,3 +1123,14 @@ QVariantList Manager::backendsVariant() const
     std::transform(d->m_backends.begin(), d->m_backends.end(), std::back_inserter(l), [](const auto &b) { return QVariant::fromValue(b); });
     return l;
 }
+
+bool Manager::eventFilter(QObject *object, QEvent *event)
+{
+    if (event->type() == QEvent::LanguageChange && object == QCoreApplication::instance() && !QCoreApplication::closingDown()) {
+        reload();
+    }
+
+    return QObject::eventFilter(object, event);
+}
+
+#include "moc_manager.cpp"
